if not HasKey("savegame.mod.vox") then
    SetInt("savegame.mod.vox", 1)
end
if not HasKey("savegame.mod.sound") then
    SetInt("savegame.mod.sound", 1)
end

----------------------------------------------------------------------------------------------------------

function randomVec()
    return Vec(math.random() - 0.5, math.random() - 0.5, math.random() - 0.5)
end


function playRandomSound(handles, pos, volume)
    local snd = handles[math.random(#handles)]
    PlaySound(snd, pos, volume)
end

function randomVecNormalize()
    return VecNormalize(randomVec())
end

function addKeyframe(anim, time, transform, easeStyle, easeDirection)
    local keyframe = {}
    keyframe.time = time
    keyframe.transform = transform or Transform()
    keyframe.easeStyle = easeStyle or 0
    keyframe.easeDirection = easeDirection or 0
    table.insert(anim.keyframes, #anim.keyframes, keyframe)
    
    if time > anim.duration then
        anim.duration = time
    end
end

function insertKeyframe(anim, time, transform, easeStyle, easeDirection)
    local keyframe = {}
    keyframe.time = time
    keyframe.transform = transform or Transform()
    keyframe.easeStyle = easeStyle or 0
    keyframe.easeDirection = easeDirection or 0
    table.insert(anim.keyframes, keyframe)
    
    if time > anim.duration then
        anim.duration = time
    end
end

function setKeyframe(anim, index, time, transform, easeStyle, easeDirection)
    local keyframe = anim.keyframes[index]
    keyframe.time = time or keyframe.time
    keyframe.transform = transform or keyframe.transform
    keyframe.easeStyle = easeStyle or keyframe.easeStyle
    keyframe.easeDirection = easeDirection or keyframe.easeDirection
end

function animation()
    local anim = {}
    anim.duration = 0
    anim.playbackSpeed = 1
    anim.keyframes = {}
    insertKeyframe(anim, 0)
    insertKeyframe(anim, 0)
    return anim
end

function playAnimation(anim, time)
    local k0 = anim.keyframes[1]
    local k1 = anim.keyframes[2]

    for i = 1, #anim.keyframes - 1 do
        k0 = anim.keyframes[i]
        k1 = anim.keyframes[i + 1]
        if k1.time > time then
            break
        end
    end

    local t = (time - k0.time) / (k1.time - k0.time)

    local pos = VecLerp(k0.transform.pos, k1.transform.pos, t)
    local rot = QuatSlerp(k0.transform.rot, k1.transform.rot, t)

    return Transform(pos, rot)
end

function setAnimationStartKeyframe(anim, transform, easeStyle, easeDirection)
    setKeyframe(anim, 1, 0, transform, easeStyle, easeDirection)
end

function setAnimationEndKeyframe(anim, transform, easeStyle, easeDirection)
    setKeyframe(anim, #anim.keyframes, anim.duration + 0.1, transform, easeStyle, easeDirection)
end

function applyImpulseBatch(position, applyPosition, velocity)
    impulseBatch = QueryAabbBodies(VecAdd(position, Vec(-1, -1, -1)), VecAdd(position, Vec(1, 1, 1)))
    impulseApplyPos = applyPosition
    impulse = velocity
end

function saberOn()
    if on then return end
    on = true
    playRandomSound(outSnds, toolPos, 1)
end

function saberOff()
    if not on then return end
    on = false
    playRandomSound(inSnds, toolPos, 1)
end

function saberSwingStart()
    if swinging then return end
    if throwing then return end

    saberOn()

    swinging = true
    clash = false
    swingT = 0

    local r = math.random(1, #swingAnimations)
    swingAnimation = swingAnimations[r]
    swingDirection = swingDirections[r]

    setAnimationStartKeyframe(swingAnimation, transform)
    setAnimationEndKeyframe(swingAnimation, idleTransform)

    playRandomSound(swingSnds, toolPos, 1)
end

function saberSwingUpdate(dt)
    targetTransform = playAnimation(swingAnimation, swingT)

    local saberTransform = TransformToParentTransform(cameraTransform, transform)
    local dir = TransformToParentVec(saberTransform, Vec(0, 0, -1))
    for i = 0.5, 1.5, 0.5 do
        local saberPos = VecAdd(saberTransform.pos, VecScale(dir, i))
        MakeHole(saberPos, 0.5, 0.5, 0.5, true)
    end

    if swingT > swingAnimation.duration + 0.1 then
        swinging = false
    end

    if not clash and swingT > 0.2 then
        clash = true
        local hit, dist, normal, shape = QueryRaycast(cameraTransform.pos, cameraDir, 2.5, 0.5)
        if hit then
            local hitPos = VecAdd(cameraTransform.pos, VecScale(cameraDir, dist))
            MakeHole(hitPos, 1, 1, 1, false)
            
            for i = 1, 2 do
                SpawnFire(VecAdd(hitPos, VecScale(randomVec(), 0.5)))
            end
            
            playRandomSound(clashSnds, toolPos, 1)

            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(5, 10)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(1)
			ParticleSticky(1)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(1,2))
                SpawnParticle(hitPos, vel, math.random(6,8))
            end
            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(4, 8)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(0, 1, "easeout")
			ParticleSticky(1, .3)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(3,5))
                SpawnParticle(hitPos, vel, math.random(5,7))
            end
            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(3, 6)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(0, 1, "easeout")
			ParticleSticky(1, .3)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(7,10))
                SpawnParticle(hitPos, vel, math.random(4,6))
            end

            velocity = VecScale(TransformToParentVec(cameraTransform, swingDirection), 500)
            applyImpulseBatch(hitPos, hitPos, velocity)
        end
    end
end

function saberThrowStart()
    if swinging then return end
    if throwing then return end

    saberOn()

    throwing = true
    throwReturn = false

    local hit, dist = QueryRaycast(cameraTransform.pos, cameraDir, 100)
    throwTarget = VecAdd(cameraTransform.pos, VecScale(cameraDir, dist))
    throwTransform = TransformCopy(cameraTransform)
    throwTransform.rot = QuatRotateQuat(throwTransform.rot, QuatEuler(0, -45, 0))

    playRandomSound(spinSnds, toolPos, 1)
end

function saberThrowUpdate(dt)

    local shapeTransform = GetShapeLocalTransform(handleShape)

    local throwPos = throwTransform.pos
    local throwRot = throwTransform.rot

    -- Saber Early Return
    if throwing == true and not InputDown("rmb") then
        if InputDown("lmb") then 
            throwReturn = true
        end
    end
    
    if throwing == true and InputDown("rmb") then
        local t = GetCameraTransform()
	    local pos = t.pos
	    local dir = TransformToParentVec(t, Vec(0, 0, -1))
	    local hit, dist, normal, shape = QueryRaycast(pos, dir, 100)
        local hitpoint = VecAdd(pos, VecScale(dir, dist))
        throwTarget = hitpoint
        if throwReturn and InputDown("rmb") then
            throwTarget = hitpoint
        end
    end

    --[[if VecLength(VecSub(throwTarget, throwPos)) < 1 then
        if throwReturn then
            throwing = false
        else
            throwReturn = true
        end
    end--]]

    if throwReturn and InputPressed("lmb") then
        throwTarget = playerPos
        sabercheck = true
    end

    if VecLength(VecSub(playerPos, throwPos)) < 3 and sabercheck == true then
        if throwReturn then
            throwing = false
        else
            sabercheck = false
        end
    end

    local up = TransformToParentVec(throwTransform, Vec(0, 1, 0))
    throwDirection = TransformToParentVec(throwTransform, Vec(0, 0, -1))
    local predThrowPos = VecAdd(throwPos, VecScale(throwDirection, 2))
    local targetDirection = VecNormalize(VecSub(throwTarget, throwPos))
    local distance = VecLength(VecSub(throwTarget, throwPos))

    throwPos = VecAdd(throwPos, VecScale(throwDirection, dt * 18)) -- Saber Flying Speed

    throwTransform.pos = throwPos

    theta = math.deg(math.acos(math.min(math.max(VecDot(throwDirection, targetDirection), -1), 1)))
    local sign = VecDot(VecCross(throwDirection, targetDirection), up)
    if (sign < 0) then
        sign = -1
    else
        sign = 1
    end
    theta = theta * sign

    throwTransform.rot = QuatSlerp(throwTransform.rot, QuatLookAt(predThrowPos, throwTarget), dt * 4) -- Saber Boomerang Pattern Size

    if GetTime() > hitT then
        local hit, dist = QueryRaycast(throwPos, throwDirection, dt * 8, 1)
        local hitpoint1 = VecAdd(throwPos, VecScale(throwDirection, dist+1))
        local hitpoint2 = VecAdd(throwPos, VecScale(throwDirection, dist+2))
        local hitpoint3 = VecAdd(throwPos, VecScale(throwDirection, dist+3))
        local hitpoint4 = VecAdd(throwPos, VecScale(throwDirection, dist-1))
        local hitpoint5 = VecAdd(throwPos, VecScale(throwDirection, dist-2))
        local hitpoint5 = VecAdd(throwPos, VecScale(throwDirection, dist-3))
        if hit then
            local hitPos = throwPos
            MakeHole(hitPos, 2, 2, 2, false)
            SpawnFire(hitpoint1)
            SpawnFire(hitpoint2)
            SpawnFire(hitpoint3)
            SpawnFire(hitpoint4)
            SpawnFire(hitpoint5)
            SpawnFire(hitpoint6)
            playRandomSound(clashSnds, hitPos, 1)

            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(20, 30)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(1)
			ParticleSticky(1)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(1,2))
                SpawnParticle(hitPos, vel, math.random(6,8))
            end
            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(15, 25)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(0, 1, "easeout")
			ParticleSticky(1, .3)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(3,5))
                SpawnParticle(hitPos, vel, math.random(5,7))
            end
            ParticleReset()
			ParticleEmissive(1, 0, "easein")
			ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
			ParticleRadius(math.random(10, 20)*.001, 0, "smooth")
			ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
			ParticleTile(4)
			ParticleCollide(0, 1, "easeout")
			ParticleSticky(1, .3)
            for i = 1, math.random(32, 64) do
                local vel = VecScale(randomVec(), math.random(7,10))
                SpawnParticle(hitPos, vel, math.random(4,6))
            end

            velocity = VecScale(throwDirection, 1000)
            applyImpulseBatch(hitPos, throwTransform.pos, velocity)

            hitT = GetTime() + 0.2
        end
    end

    PointLight(throwPos, color[1], color[2], color[3], 4)
    PlayLoop(humSnd, throwPos, 0.5)

end

function init()
   saberVoxes = {
        "MOD/vox/lightsword_red.vox",
        "MOD/vox/lightsword_green.vox",
        "MOD/vox/lightsword_blue.vox",
        "MOD/vox/lightsword_purple.vox",
        "MOD/vox/lightsword_yellow.vox",
        "MOD/vox/lightsword_orange.vox",
        "MOD/vox/lightsword_white.vox"
    }

    saberColors = {
        {1, 0, 0},
        {0, 1, 0},
        {0, 0, 1},
        {1, 0, 1},
        {1, 1, 0},
        {1, .3, 0},
        {1, 1, 1}
    }

    saberSoundGroups = {
        1, 2, 3, 2, 1, 2, 3, 2
    }

    voxIndex = GetInt("savegame.mod.vox")
    vox = saberVoxes[voxIndex]
    color = saberColors[voxIndex]
    soundGroup = saberSoundGroups[voxIndex]

    --Register tool and enable it
    RegisterTool("sithsaber", "Sith Saber", vox)
    SetBool("game.tool.sithsaber.enabled", true)
    SetFloat("game.tool.sithsaber.ammo", 10000)

    ready = 0
    fireTime = 0

    openSnd = LoadSound("MOD/snd/open.ogg")
    closeSnd = LoadSound("MOD/snd/close.ogg")
    forcepush = LoadSound("MOD/snd/forcepush.ogg")
    laserSnd = LoadLoop("MOD/snd/laser.ogg")
    hitSnd = LoadLoop("MOD/snd/hit.ogg")
    humSnd = LoadLoop("MOD/snd/hum01.ogg")
    electro = LoadLoop("MOD/snd/electro.ogg")
    electro2 = LoadSound("MOD/snd/electro2.ogg")
    lightning1 = LoadSound("MOD/snd/lightning1.ogg")
    lightning2 = LoadSound("MOD/snd/lightning2.ogg")
    lightning3 = LoadSound("MOD/snd/lightning3.ogg")
    lightning4 = LoadSound("MOD/snd/lightning4.ogg")
    lightning5 = LoadSound("MOD/snd/lightning5.ogg")
    

    swingSnds = {
        LoadSound("MOD/snd/swng01.ogg"),
        LoadSound("MOD/snd/swng02.ogg"),
        LoadSound("MOD/snd/swng03.ogg"),
        LoadSound("MOD/snd/swng04.ogg"),
        LoadSound("MOD/snd/swng05.ogg"),
        LoadSound("MOD/snd/swng06.ogg"),
        LoadSound("MOD/snd/swng07.ogg"),
        LoadSound("MOD/snd/swng08.ogg")
    }

    clashSnds = {
        LoadSound("MOD/snd/clsh01.ogg"),
        LoadSound("MOD/snd/clsh02.ogg"),
        LoadSound("MOD/snd/clsh03.ogg"),
        LoadSound("MOD/snd/clsh04.ogg"),
        LoadSound("MOD/snd/clsh05.ogg"),
        LoadSound("MOD/snd/clsh06.ogg"),
        LoadSound("MOD/snd/clsh07.ogg"),
        LoadSound("MOD/snd/clsh08.ogg")
    }

    spinSnds = {
        LoadSound("MOD/snd/spin01.ogg"),
        LoadSound("MOD/snd/spin02.ogg")
    }

    outSnds = {
        LoadSound("MOD/snd/out01.ogg"),
        LoadSound("MOD/snd/out02.ogg")
    }

    inSnds = {
        LoadSound("MOD/snd/in01.ogg"),
        LoadSound("MOD/snd/in02.ogg")
    }

    idleTransform = Transform(Vec(0.5, -0.9, -1), QuatEuler(80, 5, -5))

    swingAnimation1 = animation()
    addKeyframe(swingAnimation1, 0.1, Transform(Vec(-0.6, -0.2, -0.5), QuatEuler(30, 75, 0)))
    addKeyframe(swingAnimation1, 0.2, Transform(Vec(0.2, -0.45, -1), QuatEuler(10, 0, 0)))
    addKeyframe(swingAnimation1, 0.3, Transform(Vec(0.8, -0.6, -0.5), QuatEuler(-30, -120, 0)))

    swingAnimation2 = animation()
    addKeyframe(swingAnimation2, 0.1, Transform(Vec(0.9, -0.2, -0.5), QuatEuler(45, -75, 0)))
    addKeyframe(swingAnimation2, 0.15, Transform(Vec(0.2, -0.45, -1), QuatEuler(10, 0, 0)))
    addKeyframe(swingAnimation2, 0.25, Transform(Vec(-0.6, -0.6, -0.5), QuatEuler(-50, 120, 0)))

    swingAnimation3 = animation()
    addKeyframe(swingAnimation3, 0.1, Transform(Vec(0.2, 0.2, -0.5), QuatEuler(150, 0, -30)))
    addKeyframe(swingAnimation3, 0.15, Transform(Vec(0, -0.5, -1), QuatEuler(10, 0, 0)))
    addKeyframe(swingAnimation3, 0.25, Transform(Vec(-0.2, -1.5, -0.5), QuatEuler(-90, 0, -30)))

    swingAnimations = {
        swingAnimation1,
        swingAnimation2,
        swingAnimation3
    }

    swingDirections = {
        Vec(1, 0, 0),
        Vec(-1, 0, 0),
        Vec(0, -1, 0)
    }

    swingAnimation = swingAnimation1
    swingDirection = Vec()

    transform = Transform()

    on = true
    swinging = false
    clash = false
    throwing = false
    throwReturn = false
    swingT = 0
    hitT = 0
    onT = 0

    forceVelocity = Vec()
    torqueVelocity = Vec()

    throwDirection = Vec()
    throwTarget = Vec()
    throwTransform = Transform()

    swordOffsetTransform = Transform(Vec(-0.1, -1.1, -0.1))

    shapeLocalTransform = Transform(Vec(-0.15, -0.1, 0.45), Quat(-0.70711, 0, 0, 0.70711))

    impulseBatch = {}
    impulseApplyPos = Vec()
    impulse = Vec()

    theta = 0

    playerTransform = Transform()
    playerPos = Vec()
    cameraTransform = Transform()
    cameraDir = Vec()

    toolBody = 0
    toolTransform = Transform()
    toolPos = Vec()

    targetTransform = Transform()
end


      
function tick(dt)
    playerTransform = GetPlayerTransform()
    playerPos = VecAdd(playerTransform.pos, Vec(0, 1, 0))
    cameraTransform = GetPlayerCameraTransform()
    cameraDir = TransformToParentVec(cameraTransform, Vec(0, 0, -1))

	if GetString("game.player.tool") == "sithsaber" and GetPlayerVehicle() == 0 then

        -- Force Lightning
        if InputDown("e") then
            SetPlayerHealth(1)  
            PlayLoop(electro, toolPos, 1)
            local t = GetCameraTransform()
		    local pos = t.pos
		    local dir = TransformToParentVec(t, Vec(0, 0, -1))
		    local hit, dist, normal, shape = QueryRaycast(pos, dir, 35)
            local hitpoint = VecAdd(pos, VecScale(dir, dist))
            local hitpoint1 = VecAdd(pos, VecScale(dir, dist-1))
            local hitpoint2 = VecAdd(pos, VecScale(dir, dist+1.5))
            if math.random(1,4) == 4 then
            get_bodys(hitpoint)
            get_bodys(hipoint2)
            end
            if hit and math.random(1,3) == 3 then
                --PointLight(hitpoint, color[1], color[2], color[3], math.random(10,30)) 
                ParticleReset()
                ParticleEmissive(1, 0, "easein")
                ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
                ParticleRadius(math.random(20, 30)*.001, 0, "smooth")
                ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
                ParticleTile(4)
                ParticleCollide(1)
                ParticleSticky(1)
                for i = 1, math.random(32, 64) do
                    local vel = VecScale(randomVec(), math.random(1,2))
                    SpawnParticle(hitpoint, vel, math.random(4,6))
                end
                ParticleReset()
                ParticleEmissive(1, 0, "easein")
                ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
                ParticleRadius(math.random(15, 25)*.001, 0, "smooth")
                ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
                ParticleTile(4)
                ParticleCollide(0, 1, "easeout")
                ParticleSticky(1, .3)
                for i = 1, math.random(32, 64) do
                    local vel = VecScale(randomVec(), math.random(3,5))
                    SpawnParticle(hitpoint, vel, math.random(4,5))
                end
                ParticleReset()
                ParticleEmissive(1, 0, "easein")
                ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
                ParticleRadius(math.random(10, 20)*.001, 0, "smooth")
                ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
                ParticleTile(4)
                ParticleCollide(0, 1, "easeout")
                ParticleSticky(1, .3)
                     
                for i = 1, math.random(32, 64) do
                    local vel = VecScale(randomVec(), math.random(7,10))
                    SpawnParticle(hitpoint, vel, math.random(4,6))
                end
            end
           if hit and math.random(1,20) == 11 then 
                Explosion(hitpoint, math.random(5, 8)*.1)
                PlaySound(lightning1, toolPos, 1)
            end

            if hit and math.random(1,20) == 13 then 
                Explosion(hitpoint, math.random(5, 8)*.1) 
            end
        
            if hit and math.random(1,20) == 14 then 
                Explosion(hitpoint, math.random(5, 8)*.1)
                for i = 1, 10 do
                SpawnFire(hitpoint)
            end
        end
         
            if hit and math.random(1,20) == 15 then 
                Explosion(hitpoint, math.random(5, 8)*.1) 
            end			   
        end

          if InputPressed("e") then
               PlaySound(electro2, toolPos, 1)
            end

           if InputDown("e") and math.random(1,4) == 4 then
               if lookpos() then
               ---push(lookpos())
            end
        end


function push(pos)
QueryRequire("physical dynamic")
	local max = 8.5
    local maxDist1 = 35
    local maxMass1 = 500
    local maxMass2 = 2500
    local objectbodies = QueryAabbBodies( VecAdd(pos, Vec(-1, -1, -1)), VecAdd(pos, Vec(1, 1, 1)))
    for i = 1, #objectbodies do
        local objectbodies2 = objectbodies[i]
        if IsBodyDynamic(objectbodies2) then
            local bb, bbba = GetBodyBounds(objectbodies2)
            local direction = VecSub(VecLerp(bb, bbba, 0.5), pos)
            local distance = VecLength(direction)
            local mass = GetBodyMass(objectbodies2)
            direction = VecScale(direction, 1 / distance)
            if distance < maxDist1 and IsBodyVisible(objectbodies2, max * 35) and mass > maxMass1 and mass < maxMass2 then 
                local distScale = 1 - math.min(distance / max, 1.0)
                local vel = GetBodyVelocity(objectbodies2)
                vel = VecAdd(GetBodyVelocity(objectbodies2), VecScale(direction, 11 * distScale))
                SetBodyVelocity(objectbodies2, vel)
            end
        end
    end
end

function get_bodys(pos)
        QueryRequire("physical")
	local bods = QueryAabbBodies( VecAdd(pos, Vec(-3, -3, -3)), VecAdd(pos, Vec(3, 3, 3)))
	    for i=1, #bods do
		local body = bods[i]
		local com = GetBodyCenterOfMass(body)
		local maxMass1 = 500
        	local maxMass2 = 3500
              	local mass = GetBodyMass(body)
                local broken = IsBodyBroken(body)
                local voxelcount = GetShapeVoxelCount(shape)
                if mass < maxMass1 and mass < maxMass2 and IsBodyActive(body, 35) and IsBodyVisible(body, 35) then 
		local t = TransformToParentPoint(GetBodyTransform(body), com)
		rv = VecAdd(t, Vec(math.random(-2, 2), math.random(-2, 2), math.random(-2, 2)))
                rv2 = VecAdd(t, Vec(math.random(-2, 2), math.random(-2, 2), math.random(-2, 2)))
                
                if math.random(1,4) == 4 and IsBodyActive(body, 35) and IsBodyVisible(body, 35) then
                SpawnFire(rv)
                QueryRejectVehicle(vehicle)
                MakeHole(rv, .9, .9, .9)                
                end

        local fwd = TransformToParentVec(com, Vec(0, 0, -1))
		local hit, dist, normal, shape = QueryRaycast(t, fwd, 3)
								
		local e = VecAdd(t, VecScale(fwd, dist))

		local last = pos
 
         if IsBodyActive(body, 40) and IsBodyVisible(body, 40) then	 
		for i=1, 12 do
			local t = i/10
			local p = VecLerp(pos, e, t)
			p = VecAdd(p, VecScale(VecNormalize(Vec(math.random(-3,3), math.random(-3,3), math.random(-3,3))), 0.2*t))
                        DrawLine(last, p, 1.0, 1.0, 1.0, 1)
                        if math.random(1,3) == 3 then
                        PointLight(p, 1.0, 1.0, 1.0, 0.06)
                        end
			last = p

                        if math.random(1,20) == 15 then
                        ParticleReset()
                	ParticleEmissive(1, 0, "easein")
                	ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
                	ParticleRadius(math.random(20, 30)*.001, 0, "smooth")
                	ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
                	ParticleTile(4)
                	ParticleCollide(1)
                	ParticleSticky(1)
                	for i = 1, math.random(1, 2) do
                    	local vel = VecScale(randomVec(), math.random(11,12))
                    	SpawnParticle(p, vel, 0.2)
                	end
                	
                        ParticleReset()
                	ParticleEmissive(1, 0, "easein")
                	ParticleGravity(math.random(5, 8)*-1, math.random(8, 10)*-1, "smooth")
                	ParticleRadius(math.random(15, 25)*.001, 0, "smooth")
                	ParticleColor(1,math.random(75, 85)*0.01,math.random(55, 65)*0.01, 1,.4,0, "smooth")          
                	ParticleTile(4)
                	ParticleCollide(0, 1, "easeout")
                	ParticleSticky(1, .3)
                	for i = 1, math.random(1, 2) do
                    	local vel = VecScale(randomVec(), math.random(13,15))
                    	SpawnParticle(p, vel, 0.2)
                	end
           	end
		end
	end
   end
end
end

function get_bodys1(pos)
	QueryRejectVehicle(vehicle)
	QueryRequire("physical")
	local bods = QueryAabbBodies( VecAdd(pos, Vec(-2, -2, -2)), VecAdd(pos, Vec(2, 2, 2)))
	for i=1, #bods do
		local body = bods[i]
		local com = GetBodyCenterOfMass(body)
                local mass = GetBodyMass(body)
                local voxelCount = GetShapeVoxelCount(shape)
		local t = TransformToParentPoint(GetBodyTransform(body), com)
		rv1 = VecAdd(t, Vec(math.random(-2, 2), math.random(-2, 2), math.random(-2, 2)))
               
        MakeHole(rv1, .8, .8, .8)
                                     
		local fwd = TransformToParentVec(com, Vec(0, 0, -1))
		local hit, dist, normal, shape = QueryRaycast(t, fwd, .15)
	end
end

function lookpos()
  local t = GetCameraTransform()
  local transform = TransformToParentVec(t, Vec(0, 0, -1))
  hit, dist, normal, shape = QueryRaycast(t.pos, transform, 35)
  if hit then
      local pos = VecAdd(t.pos, VecScale(transform, dist))
      return pos
    else
      return false
    end
end
   
		-- Force Lift/Push/Pull
		if InputDown("q") and not InputDown("w") and not InputDown("s") then

            PlaySound(forcepush, toolPos, .6)
            strength = 1	--Strength of Force Lift
            maxMass = 1000000	--The maximum mass for a body to be affected
            maxDist = 25	--The maximum distance for bodies to be affected

			--Get all physical and dynamic bodies in front of camera
			local t = GetCameraTransform()
			local c = TransformToParentPoint(t, Vec(0, 0, -maxDist/1.9))
            local f = TransformToParentPoint(t, Vec(0, 0, -1))
			local mi = VecAdd(c, Vec(-maxDist/2, -maxDist/2, -maxDist/2))
			local ma = VecAdd(c, Vec(maxDist/2, maxDist/2, maxDist/2))
			QueryRequire("physical dynamic")
			local bodies = QueryAabbBodies(mi, ma)

			--Loop through bodies and push them
			for i=1,#bodies do
				local b = bodies[i]

				--Compute body center point and distance
				local bmi, bma = GetBodyBounds(b)
				local bc = VecLerp(bmi, bma, 0.5)
				local dir = VecSub(bc, t.pos)
				local dist = VecLength(dir)
				dir = VecScale(dir, 1.0/dist)

				--Get body mass
				local mass = GetBodyMass(b)
				
				--Check if body is should be affected
				if dist < maxDist and mass < maxMass then
					--Make sure direction is always pointing slightly upwards
					--dir[2] = .5
					--dir = VecNormalize(dir)
                    dir = Vec(0,.6,0)
					--Compute how much velocity to add
					local massScale = 1 - math.min(mass/maxMass, 1.0)
					local distScale = 1 - math.min(dist/maxDist, 1.0)
					local add = VecScale(dir, strength * massScale * distScale)
					
					--Add velocity to body
					local vel = GetBodyVelocity(b)
					vel = VecAdd(vel, add)
					SetBodyVelocity(b, vel)
				end
			end
            -- Blue Force Effect
            ParticleReset()
            ParticleAlpha(.1, 0.05, "smooth")
			ParticleEmissive(1, .5, "smooth")
			ParticleRadius(math.random(10, 20)*.1, 0, "smooth")
            --ParticleColor(color[1], color[2], color[3]) 
			ParticleColor(0,0,math.random(80, 100)*0.01, 0,0,math.random(70, 90)*0.01, "smooth")          
			ParticleTile(14)
			ParticleCollide(0, 1, "smooth")
            for i = 1, math.random(4, 8) do
                local vel = VecScale(randomVec(), math.random(7,10))
                SpawnParticle(f, vel, math.random(10,20)*.1)
            end
        end

        if InputDown("q") and InputDown("w") and not InputDown("s") then

            PlaySound(forcepush, toolPos, .6)
            strength = .8	--Strength of Force Push
            maxMass = 1000000	--The maximum mass for a body to be affected
            maxDist = 25	--The maximum distance for bodies to be affected

			--Get all physical and dynamic bodies in front of camera
			local t = GetCameraTransform()
			local c = TransformToParentPoint(t, Vec(0, 0, -maxDist/1.9))
            local f = TransformToParentPoint(t, Vec(0, 0, -2.5))
			local mi = VecAdd(c, Vec(-maxDist/2, -maxDist/2, -maxDist/2))
			local ma = VecAdd(c, Vec(maxDist/2, maxDist/2, maxDist/2))
			QueryRequire("physical dynamic")
			local bodies = QueryAabbBodies(mi, ma)

			--Loop through bodies and push them
			for i=1,#bodies do
				local b = bodies[i]

				--Compute body center point and distance
				local bmi, bma = GetBodyBounds(b)
				local bc = VecLerp(bmi, bma, 0.5)
				local dir = VecSub(bc, t.pos)
				local dist = VecLength(dir)
				dir = VecScale(dir, 1.0/dist)

				--Get body mass
				local mass = GetBodyMass(b)
				
				--Check if body is should be affected
				if dist < maxDist and mass < maxMass then
					--Make sure direction is always pointing slightly upwards
					dir[2] = 1
					dir = VecNormalize(dir)
			
					--Compute how much velocity to add
					local massScale = 1 - math.min(mass/maxMass, 1.0)
					local distScale = 1 - math.min(dist/maxDist, 1.0)
					local add = VecScale(dir, strength * massScale * distScale)
					
					--Add velocity to body
					local vel = GetBodyVelocity(b)
					vel = VecAdd(vel, add)
					SetBodyVelocity(b, vel)
				end
			end
            -- Blue Force Effect
            ParticleReset()
            ParticleAlpha(.1, 0.05, "smooth")
			ParticleEmissive(1, .5, "smooth")
			ParticleRadius(math.random(10, 20)*.1, 0, "smooth")
            --ParticleColor(color[1], color[2], color[3])
			ParticleColor(0,0,math.random(80, 100)*0.01, 0,0,math.random(70, 90)*0.01, "smooth")          
			ParticleTile(14)
			ParticleCollide(0, 1, "smooth")
            for i = 1, math.random(4, 8) do
                local vel = VecScale(randomVec(), math.random(7,10))
                SpawnParticle(f, vel, math.random(10,20)*.1)
            end
        end

        if InputDown("q") and InputDown("s") and not InputDown("w") then

            PlaySound(forcepush, toolPos, .6)
            strength = .8	--Strength of Force Pull
            maxMass = 1000000	--The maximum mass for a body to be affected
            maxDist = 25	--The maximum distance for bodies to be affected

			--Get all physical and dynamic bodies in front of camera
			local t = GetCameraTransform()
			local c = TransformToParentPoint(t, Vec(0, 0, -maxDist/1.9))
            local f = TransformToParentPoint(t, Vec(0, 0, 1))
			local mi = VecAdd(c, Vec(-maxDist/2, -maxDist/2, -maxDist/2))
			local ma = VecAdd(c, Vec(maxDist/2, maxDist/2, maxDist/2))
			QueryRequire("physical dynamic")
			local bodies = QueryAabbBodies(mi, ma)

			--Loop through bodies and push them
			for i=1,#bodies do
				local b = bodies[i]

				--Compute body center point and distance
				local bmi, bma = GetBodyBounds(b)
				local bc = VecLerp(bmi, bma, 0.5)
				local dir = VecSub(bc, t.pos)
				local dist = VecLength(dir)
				dir = VecScale(dir, 1.0/dist)

				--Get body mass
				local mass = GetBodyMass(b)
				
				--Check if body is should be affected
				if dist < maxDist and mass < maxMass then
					--Make sure direction is always pointing slightly upwards
					dir[2] = -1
					dir = VecNormalize(dir)
			
					--Compute how much velocity to add
					local massScale = 1 - math.min(mass/maxMass, 1.0)
					local distScale = 1 - math.min(dist/maxDist, 1.0)
					local add = VecScale(dir, strength * massScale * distScale)
					
					--Add velocity to body
					local vel = GetBodyVelocity(b)
					vel = VecSub(vel, add)
					SetBodyVelocity(b, vel)
				end
			end
            -- Blue Force Effect
            ParticleReset()
            ParticleAlpha(.1, 0.05, "smooth")
			ParticleEmissive(1, .5, "smooth")
			ParticleRadius(math.random(10, 20)*.1, 0, "smooth")
            --ParticleColor(color[1], color[2], color[3])
			ParticleColor(0,0,math.random(80, 100)*0.01, 0,0,math.random(70, 90)*0.01, "smooth")          
			ParticleTile(14)
			ParticleCollide(0, 1, "smooth")
            for i = 1, math.random(4, 8) do
                local vel = VecScale(randomVec(), math.random(7,10))
                SpawnParticle(f, vel, math.random(10,20)*.1)
            end
        end
    end

	--Check if light saber is selected
	if GetString("game.player.tool") == "sithsaber" then        
        toolBody = GetToolBody()
        toolTransform = GetBodyTransform(toolBody)
        toolPos = toolTransform.pos

        local shapes = GetBodyShapes(toolBody)
          
        local handleShape = shapes[1]
        local bladeShape = shapes[2]

        SetTag(handleShape, "nocull", "true")
        SetTag(bladeShape, "nocull", "true")

        targetTransform = idleTransform
        
		--Check if tool is firing
        if GetBool("game.player.canusetool") then
            if InputPressed("lmb") then
                saberSwingStart()
            end
            if InputPressed("rmb") then
                saberThrowStart()
            end
            if not swinging and not throwing and InputPressed("c") then
                if on then
                    saberOff()
                else
                    saberOn()
                end
            end
            if swinging then
                saberSwingUpdate(dt)
            end
		end

        if throwing then
            local rotationTransform = TransformToParentTransform(throwTransform, Transform(Vec(), QuatEuler(-90, GetTime() * 1200, 0)))
            rotationTransform.rot = QuatRotateQuat(QuatAxisAngle(throwDirection, -theta), rotationTransform.rot)
            local swordThrowTransform = TransformToParentTransform(rotationTransform, swordOffsetTransform)
            local localTransform = TransformToLocalTransform(toolTransform, swordThrowTransform)

            SetShapeLocalTransform(handleShape, localTransform)
            SetShapeLocalTransform(bladeShape, TransformToParentTransform(localTransform, Transform(Vec(0.05, 0.5, 0.05))))
        else
            local pos = transform.pos
            local rot = transform.rot

            local bladeLength = 2 * onT - 1.5

            transform = Transform(VecLerp(pos, targetTransform.pos, dt * 20), QuatSlerp(rot, targetTransform.rot, dt * 20))
            
            SetShapeLocalTransform(handleShape, shapeLocalTransform)
            SetShapeLocalTransform(bladeShape, TransformToParentTransform(shapeLocalTransform, Transform(Vec(0.05, bladeLength, 0.05))))
            SetToolTransform(transform, 1)

            PointLight(VecAdd(toolPos, TransformToParentVec(toolTransform, Vec(0, 1, 0))), color[1], color[2], color[3], 4 * onT)
            PlayLoop(humSnd, toolPos, onT)

        end
	end

    if throwing then
        saberThrowUpdate(dt)
    end

    if on then
        onT = math.min(onT + dt * 4, 1)
    else
        onT = math.max(onT - dt * 4, 0)
    end

    if swinging then
        swingT = swingT + dt
    end
end

function update(dt)
    if #impulseBatch > 0 then
        for i, body in pairs(impulseBatch) do
            local mass = GetBodyMass(body)
            local bodyImpulse = VecScale(impulse, 1 / (mass + 50) * mass)
            ApplyBodyImpulse(body, impulseApplyPos, bodyImpulse)
        end
        impulseBatch = {}
    end
end
function draw() -- Force Lightning Effects and Hands
	if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") and math.random(1,20) == 10 then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/shock1.png")
		UiPop()
	end
    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") and math.random(1,20) == 11 then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/shock2.png")
		UiPop()
	end
    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") and math.random(1,20) == 12 then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/shock3.png")
		UiPop()
	end
    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") and math.random(1,20) == 13 then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/shock4.png")
		UiPop()
	end
     if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") and math.random(1,20) == 14 then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/shock5.png")
		UiPop()
	end
    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("q") and not InputDown("e") then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/hand_force.png")
		UiPop()
	end
    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("e") then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/hand_lightning.png")
		UiPop()
	end

    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("z") then
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/hand_lightning.png")
		UiPop()
	end

    if GetString("game.player.tool") == "sithsaber" and GetBool("game.player.canusetool") and 
        InputDown("r") then
        SetPlayerHealth(1)
		UiPush()
			UiTranslate(UiCenter(), UiMiddle())
			UiAlign("center middle") 
			UiImage("img/hand_lightning.png")
		UiPop()
	end
end